#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2017-05-15
#
#  https://developer.cisco.com/docs/support-apis/
#
# 2021-07-23: rewrite for python 3.8
#
import json
from cisco_live_cycle_utils import (
    configure_logger,
    log_message,
    expand_path,
    get_ids_from_dir,
    remove_ids_from_list,
    refresh_ids_from_dir,
    remove_ids_from_dir,
    sleep_random,
)
from ciscoapi import (
    AccessToken,
    Settings,
    get_eox_by_pid,
    get_eox_by_serials,
)


# split pids in known and unknown eox state
def split_pids(eoxr):
    eox_known = []
    eox_unkown = []

    for response in eoxr:
        EOXRecord = response.get('EOXRecord')
        #        PaginationResponseRecord = response.get('PaginationResponseRecord')
        #        PageIndex = PaginationResponseRecord.get('PageIndex')
        #        LastIndex = PaginationResponseRecord.get('LastIndex')
        #        PageRecords = PaginationResponseRecord.get('PageRecords')
        #        TotalRecords = PaginationResponseRecord.get('TotalRecords')
        if EOXRecord is not None:
            for PID in EOXRecord:
                if PID.get('EOLProductID') != '':
                    eox_known.append(PID)
                    log_message(message=f'EOLProductID  : {PID.get("EOLProductID")}')
                else:
                    eox_unkown.append(PID)
                    log_message(f'EOXInputValue : {PID.get("EOXInputValue")}')
    return {'eox_known': eox_known, 'eox_unknown': eox_unkown}


# split serials, expects a list of EoX Records from get EoX by serial
def split_serials(eoxr):
    serials = []

    for PID in eoxr:
        EOLProductID = PID.get('EOLProductID')
        log_message(f'serial split Eox: found PID: {EOLProductID}')
        EOXInputValue = PID.get('EOXInputValue').split(',')
        for serial in EOXInputValue:
            log_message(f'found Serial: {serial}')
            eox_serial = PID.copy()
            eox_serial.update({'EOXInputValue': serial})
            serials.append(eox_serial)
            log_message(f'Serial EoX: {eox_serial}')

    return serials


# save EoX records to file by PID,
# expects a list of EoX Records from Cisco EoX API 5.0,
# returns a list of saved PIDs
def save_eox(eox, path):
    saved = []

    for PID in eox:
        EOLProductID = PID.get('EOLProductID')
        # EOLProductID is empty for EoX Records with unknown EoX state (error or not announced)
        if EOLProductID == '':
            EOLProductID = PID.get('EOXInputValue')

        if EOLProductID:
            with open(path + (EOLProductID.replace('/', '_')), 'w') as f:
                json.dump(PID, f)
                saved.append(EOLProductID)

    return saved


def save_serials(eox, path):
    """
    Saves EoX records to file by serial number.
    Args:
        eox: List of EoX Records from Cisco EoX API 5.0
        path: file path where to save the EoX records

    Returns: List of serial numbers of saved EoX records

    """

    saved = []

    for serial in eox:
        EOXInputValue = serial.get('EOXInputValue')
        if EOXInputValue:
            with open(path + EOXInputValue, 'w') as f:
                json.dump(serial, f)
            saved.append(EOXInputValue)

    return saved


def main():
    settings = Settings()
    print(settings.proxies)
    access_token = AccessToken(settings.client_id, settings.client_secret, settings.proxies)
    configure_logger(log_level=settings.log_level)

    eox_path = settings.base_path + '/EoX'
    path_found = eox_path + '/found'
    path_not_found = eox_path + '/not_found'
    path_request = eox_path + '/request'

    path_request_pid = expand_path(path_request + '/pid')
    path_found_pid = expand_path(path_found + '/pid')
    path_not_found_pid = expand_path(path_not_found + '/pid')

    path_request_ser = expand_path(path_request + '/ser')
    path_found_ser = expand_path(path_found + '/ser')
    path_not_found_ser = expand_path(path_not_found + '/ser')

    # create list of PIDs to request EoX status for
    pids = get_ids_from_dir(path_request_pid)
    log_message(f'pid requests : {pids}')
    # remove already known PIDs from list
    pids = remove_ids_from_list(pids, path_found_pid)
    log_message(f'pid requests : {pids}')
    # remove PIDs already requested with unknown EoX status from list
    pids = remove_ids_from_list(pids, path_not_found_pid)
    log_message(f'pid requests : {pids}')

    # refresh PIDs after 30 days by default
    pids = refresh_ids_from_dir(path_not_found_pid, settings.eox_refresh_unknown, pids, True)
    log_message(f'pid requests : {pids}')
    pids = refresh_ids_from_dir(path_found_pid, settings.eox_refresh_known, pids, False)
    log_message(f'pid requests : {pids}')

    # create list of serial numbers to request EoX status for
    serials = get_ids_from_dir(path_request_ser)
    log_message(f'ser requests : {serials}')
    # remove already known serials from list
    serials = remove_ids_from_list(serials, path_found_ser)
    log_message(f'ser requests : {serials}')
    # remove serials already requested with unknown EoX status from list
    serials = remove_ids_from_list(serials, path_not_found_ser)
    log_message(f'ser requests : {serials}')

    # refresh serials after 30 days by default
    serials = refresh_ids_from_dir(path_not_found_ser, settings.eox_refresh_unknown, serials, True)
    log_message(f'ser requests : {serials}')
    serials = refresh_ids_from_dir(path_found_ser, settings.eox_refresh_known, serials, False)
    log_message(f'ser requests : {serials}')

    if pids == [] and serials == []:
        log_message('all list are empty. Do nothing.')
        return

    # wait random time after startup (load spread)
    if settings.wait_after_start:
        sleep_random(settings.max_wait_time)

    if pids is not []:
        eox = get_eox_by_pid(pids=pids, access_token=access_token, settings=settings)

        # split eox records in a list of known and unknown pid records
        eox = split_pids(eox)

        # save known pid reports
        pids = save_eox(eox.get('eox_known'), path_found_pid)
        # delete requests for known pids
        remove_ids_from_dir(pids, path_request_pid)

        # save unknown pid reports
        pids = save_eox(eox.get('eox_unknown'), path_not_found_pid)
        # delete requests for unknown pids
        remove_ids_from_dir(pids, path_request_pid)
        # delete pids from known were the status changed to unknown
        remove_ids_from_dir(pids, path_found_pid)

    if serials is not []:
        eox = get_eox_by_serials(serials=serials, access_token=access_token, settings=settings)
        log_message(f'eox by ser: {eox}')

        # split eox records in a list of known and unknown pid records
        eox = split_pids(eox)

        # split EoX records for known PIDs in one entry per serial
        serials = split_serials(eox.get('eox_known'))
        # save EoX records for serials with known EoX state
        serials = save_serials(serials, path_found_ser)
        log_message(f'EoX Serials: known: {serials}')
        # delete requests for known serials
        remove_ids_from_dir(serials, path_request_ser)

        # split EoX records for unknown PIDs in one entry per serial
        serials = split_serials(eox.get('eox_unknown'))
        # save EoX records for serials with known EoX state
        serials = save_serials(serials, path_not_found_ser)
        # delete requests for unknown serials
        remove_ids_from_dir(serials, path_request_ser)
        # delete serials from known were the status changed to unknown
        remove_ids_from_dir(serials, path_found_ser)


main()
